C
      SUBROUTINE COLRD1(MP, NP, DP, P, LDP1, LDP2, DR, DU, R, LDR1,
     *                  LDR2, U, LDU1, LDU2, ZERCOL, MU, S, SK, A, LDA,
     *                  AB, LDAB, Q, LDQ, Y, LDY, YI, GAMC, LDG, W,
     *                  TOL, IERR)
C
C     PURPOSE
C
C     To compute for a given polynomial matrix
C                                                    dp-1            dp
C        P(s) = P(0) + P(1) * s + . . . + P(dp-1) * s     + P(dp) * s  ,
C
C     which is not column reduced, a unimodular polynomial matrix U(s)
C     such that R(s) = P(s) * U(s) is column reduced.
C     REMARK: This auxiliary routine is intended to be called only from
C             the routine COLRED.
C
C     ARGUMENTS IN
C
C        MP - INTEGER.
C            The number of rows of the polynomial matrix P(s).
C            MP >= 1.
C        NP - INTEGER.
C            The number of columns of the polynomial matrix P(s).
C            NP >= 1.
C        DP - INTEGER.
C            The degree of the polynomial matrix P(s).
C            DP >= 0.
C        P - DOUBLE PRECISION array of DIMENSION (LDP1,LDP2,DP+1).
C            The leading MP by NP by (DP+1) part of this array must
C            contain the coefficients of the polynomial matrix P(s).
C            Specifically, P(i,j,k) must contain the coefficient of
C            s**(k-1) of the polynomial which is the (i,j)-th element
C            of P(s), where i = 1,2,...,MP, j = 1,2,...,NP and
C            k = 1,2,...,DP+1.
C        LDP1 - INTEGER.
C            The leading dimension of array P as declared in the calling
C            program.
C            LDP1 >= MP.
C        LDP2 - INTEGER.
C            The second dimension of array P as declared in the calling
C            program.
C            LDP2 >= NP.
C
C     ARGUMENTS OUT
C
C        DR - INTEGER.
C            The degree of the column reduced polynomial matrix R(s).
C        DU - INTEGER.
C            The degree of the unimodular polynomial matrix U(s).
C        R - DOUBLE PRECISION array of DIMENSION (LDR1,LDR2,DP+1).
C            The leading MP by NP by (DR+1) part of this array contains
C            the coefficients of the column reduced polynomial matrix
C            R(s). Specifically, R(i,j,k) contains the coefficient of
C            s**(k-1) of the polynomial which is the (i,j)-th element of
C            R(s), where i = 1,2,...,MP, j = 1,2,...,NP and
C            k = 1,2,...,DR+1.
C        LDR1 - INTEGER.
C            The leading dimension of array R as declared in the calling
C            program.
C            LDR1 >= MP.
C        LDR2 - INTEGER.
C            The second dimension of array R as declared in the calling
C            program.
C            LDR2 >= NP.
C        U - DOUBLE PRECISION array of DIMENSION (LDU1,LDU2,NP*DP+1).
C            The leading NP by NP by (DU+1) part of this array contains
C            the coefficients of the unimodular polynomial matrix U(s).
C            Specifically, U(i,j,k) contains the coefficient of s**(k-1)
C            of the polynomial which is the (i,j)-th element of U(s),
C            where i = 1,2,...,NP, j = 1,2,...,NP and k = 1,2,...,DU+1.
C        LDU1 - INTEGER.
C            The leading dimension of array U as declared in the calling
C            program.
C            LDU1 >= NP.
C        LDU2 - INTEGER.
C            The second dimension of array U as declared in the calling
C            program.
C            LDU2 >= NP.
C        ZERCOL - LOGICAL array of DIMENSION at least (NP).
C            If ZERCOL(j) = .TRUE. then the j-th column of R(s) is zero;
C            otherwise the j-th column belongs to R1(s) (see METHOD).
C
C        WORKSPACE
C
C        MU - INTEGER array of DIMENSION at least (mamax+1),
C            where mamax = (NP * DP + 1) * MP.
C            On exit, this array contains the row indices of the left
C            upper elements of the right invertible diagonal submatrices
C            of A'.
C        S - INTEGER array of DIMENSION at least (mamax).
C            On exit, this array contains the column indices of the
C            pivots of A', that is A transformed in upper staircase
C            form.
C        SK - INTEGER array of DIMENSION at least (2*NP).
C        A - DOUBLE PRECISION array of DIMENSION (LDA,namax),
C            where namax = mamax + NP.
C            On exit, the upper block diagonal part of this array
C            contains the transformed matrix A', the lower part contains
C            the vectors of the Householder transformations.
C        LDA - INTEGER.
C            The leading dimension of array A as declared in the calling
C            program.
C            LDA >= mamax + 1.
C        AB - DOUBLE PRECISION array of DIMENSION (LDAB,namax).
C            Array in which the transformed matrix Ab is saved.
C        LDAB - INTEGER.
C            The leading dimension of array AB as declared in the
C            calling program.
C            LDAB >= mamax + 1.
C        Q - DOUBLE PRECISION array of DIMENSION (LDQ,NP).
C        LDQ - INTEGER.
C            The leading dimension of array Q as declared in the calling
C            program.
C            LDQ >= MP.
C        Y - DOUBLE PRECISION array of DIMENSION (LDY,NP*DP+2).
C            Array in which a null vector of sA - E or sA' - E is
C            stored.
C        LDY - INTEGER.
C            The leading dimension of array Y as declared in the calling
C            program.
C            LDA >= namax.
C        YI - DOUBLE PRECISION array of DIMENSION at least (NP).
C        GAMC - DOUBLE PRECISION array of DIMENSION (LDG,NP)
C            On exit, this array contains the leading column coefficient
C            matrix of R(s), which has full column rank.
C        LDG - INTEGER.
C            The leading dimension of array GAMC as declared in the
C            calling program.
C            LDG >= MP.
C        W - DOUBLE PRECISION array of DIMENSION (mamax+namax).
C
C     TOLERANCES
C
C        TOL - DOUBLE PRECISION.
C            A tolerance below which matrix elements are considered to
C            be zero.
C
C     ERROR INDICATOR
C
C        IERR - INTEGER.
C           Unless the routine detects an error (see next section),
C           IERR contains 0 on exit.
C
C     WARNINGS AND ERRORS DETECTED BY THE ROUTINE
C
C        IERR = 2 : No column reduced R(s) has been found for the
C                   maximum b, (see METHOD).
C        IERR = 3 : The computation of a null vector has failed,
C                   because a diagonal block of A' is not right
C                   invertible.
C        IERR = 4 : The computation of R(s) has failed, because a
C                   computed null vector is not s**B times a polynomial
C                   vector.
C
C     METHOD
C
C     Let GAMC(P) be the constant matrix such that each of its columns
C     contains the coefficients of the highest power of s occurring in
C     the corresponding column of P(s), the so-called leading column
C     coefficient matrix. Then P(s) is called column reduced if there
C     exists a permutation matrix T such that P(s) = ( Z , P1(s) ) * T,
C     where Z is a zero matrix and GAMC(P1) has full column rank.
C
C     Let (U(s),Z(s))' be a minimal polynomial basis (MPB) for
C          b
C     Ker(s P(s), -I), for some b > 0. It has been proved, see [1], that
C     if b is greater than d'c(P), the sum of all but the smallest
C                                                                -b
C     column degrees of P(s), then U(s) is unimodular and R(s) = s  Z(s)
C     is column reduced and P(s) * U(s) = R(s).
C     The routine computes an MPB for b = 1,2,... and checks for each b
C     whether R(s) is column reduced, i.e. whether GAMC(R1) has full
C     column rank. The algorithm finishes with U(s) and R(s) as soon as
C     R(s) is column reduced.
C
C     REFERENCES
C
C     [1] Neven, W.H.L. and Praagman, C.
C         Column Reduction of Polynomial Matrices.
C         Linear Algebra and its Applications 188, 189, pp. 569-589,
C         1993.
C
C     CONTRIBUTORS
C
C        A.J. Geurts (Eindhoven University of Technology).
C        C. Praagman (University of Groningen).
C
C     REVISIONS
C
C        1993, November 8.
C        1996, May 21.
C
      IMPLICIT NONE
C     .. Parameters ..
      DOUBLE PRECISION ZERO, ONE
      PARAMETER (ZERO = 0.0D0, ONE = 1.0D0)
C     .. Scalar Arguments ..
      INTEGER MP, NP, DP, LDP1, LDP2, DR, DU, LDR1, LDR2, LDU1, LDU2,
     *        LDA, LDAB, LDQ, LDY, LDG, IERR
      DOUBLE PRECISION TOL
C     .. Array Arguments ..
      INTEGER MU(*), S(*), SK(*)
      DOUBLE PRECISION P(LDP1,LDP2,*), R(LDR1,LDR2,*), U(LDU1,LDU2,*),
     *                 A(LDA,*), AB(LDAB,*), Q(LDQ,*), Y(LDY,*), YI(*),
     *                 GAMC(LDG,*),  W(*)
      LOGICAL ZERCOL(*)
C     .. Local Scalars ..
      INTEGER BMAX, B, I, IC, IR, IND, NNV, NNVB, NCR1, J, K, KK,
     *        L, C1AI, DUB, DUR1, MA, NA, MAI, NAI, MUI
      DOUBLE PRECISION NORM, TAU
      LOGICAL COLRDC
C     .. External Subroutines/Functions ..
      EXTERNAL CKGAMC, COMPTV, COMPTY, COMPYI, DCOPY, DGEMV, DGER,
     *         DLACPY, DLAVEC, DLASET, DNRM2, HHTRAN, MKPENC
      DOUBLE PRECISION DNRM2
      LOGICAL CKGAMC
C     .. Intrinsic Functions ..
      INTRINSIC MAX
C
C     .. Executable Statements ..
C
      BMAX = (NP-1) * DP + 1
      CALL MKPENC(MP, NP, DP, P, LDP1, LDP2, AB, LDAB, MA, NA)
      NNVB = 0
      DUB = -1
      DO 10 K = 1, NP * DP + 1
         CALL DLASET('G', NP, NP, ZERO, ZERO, U(1,1,K), LDU1)
   10 CONTINUE
      COLRDC = .FALSE.
      MU(1) = 1
      B = 0
C     WHILE (.NOT.COLRDC and B < BMAX) DO
   20 IF (.NOT.COLRDC .AND. B.LT.BMAX) THEN
         B = B + 1
C
C        Initialization of A.
C
         IF (B .EQ. 1) THEN
            CALL DLACPY('G', MA, NA, AB, LDAB, A, LDA)
            CALL DLAVEC(NA, ZERO, A(MA+1,1), LDA)
         ELSE
            CALL DLACPY('G', MA+1, NA, AB, LDAB, A, LDA)
            J = NP + MU(B-1)
            L = MA - MP - MU(B-1) + 1
            CALL DLASET('G', MP, J-1, ZERO, ZERO, A(MA+2,1), LDA)
            CALL DLASET('G', MP+1, L, ZERO, ZERO, A(MA+1,J), LDA)
            CALL DLASET('G', MP+1, MP, ZERO, ONE, A(MA+1,L+J), LDA)
            MA = MA + MP
            CALL DLASET('G', MA+1, MP, ZERO, ZERO, A(1,NA+1), LDA)
            NA = NA + MP
            DO 30 K = MU(B-1), MU(B) - 1
               TAU = A(K+1,S(K))
               IF (TAU .NE. ZERO) THEN
                  J = NP + K
                  L = MA - MP - K + 1
                  W(MA+1) = ONE
                  CALL DCOPY(L-1, A(K+2,S(K)), 1, W(MA+2), 1)
                  CALL DGEMV('N', MP, L, ONE, A(MA-MP+1,J), LDA,
     *                        W(MA+1), 1, ZERO, W, 1)
                  CALL DGER(MP, L, -TAU, W, 1, W(MA+1), 1,
     *                      A(MA-MP+1,J), LDA)
               END IF
   30       CONTINUE
         END IF
C
         I = B
         DU = DUB
         NNV = NNVB
         DR = -1
         DO 40 K = 1, DU + 1
            CALL DLASET('G', NP, NP-NNV, ZERO, ZERO, U(1,NNV+1,K),
     *                  LDU1)
   40    CONTINUE
         DO 50 K = 1, DP + 1
            CALL DLASET('G', MP, NP, ZERO, ZERO, R(1,1,K), LDR1)
   50    CONTINUE
         NCR1 = 0
         COLRDC = .TRUE.
C
C        NNV is the number of already found null vectors. With these
C        null vectors correspond the MP by NNV column reduced matrix
C        R(s). NCR1 is the number of columns in R1(s).
C
C        WHILE (NNV < NP, which means that not all null vectors of
C               sA - E  have been found, and R(s) is column reduced) DO
   60    IF ((NNV.LT.NP) .AND. COLRDC) THEN
C
C           Determine A(i,i), the i(th) right invertible diagonal block.
C           The left upper element of A(i,i) is A(MUI,C1AI) and the row
C           and column dimensions are MAI and NAI, respectively.
C
            MUI = MU(I)
            MAI = 0
            IF (I .EQ. 1) THEN
               C1AI = 1
               NAI = NP
            ELSE
               C1AI = MU(I-1) + NP
               NAI = MU(I) - MU(I-1)
            END IF
C
C           Compute the null vectors of A(i,i) and the corresponding
C           nullvectors of sA - E one by one, append the appropriate
C           parts of a computed null vector to U(s) and R(s) and check
C           whether R(s) remains column reduced.
C
            K = 1
C           WHILE (K <= NAI and R(s) still column reduced) DO
   70       IF (K.LE.NAI .AND. COLRDC) THEN
               IR = MUI + MAI
               IC = C1AI + K - 1
               L = MA - IR + 1
               IF (L .GT. 0) THEN
                  NORM = DNRM2(L, A(IR,IC), 1)
               ELSE
                  NORM = ZERO
               END IF
               IF (NORM .GE. TOL) THEN
C
C                 Generate and apply the Householder transformation for
C                 the IC-th column of A.
C
                  CALL HHTRAN(MA, NA, IC, IR, A, LDA, W, W(MA+1))
                  CALL DCOPY(L, W, 1, A(IR+1,IC), 1)
                  S(MUI+MAI) = IC
                  MAI = MAI + 1
                  SK(MAI) = IC - C1AI + 1
               ELSE
C
C                 Compute the null vector Y of sA - E corresponding
C                 to the IC-th column of A.
C
                  IF (L .GT. 0) THEN
                     CALL DLAVEC(L, ZERO, A(IR,IC), 1)
                  END IF
                  IF (MAI .GT. 0) THEN
                     CALL DCOPY(MAI, A(MUI,IC), 1, W, 1)
                     CALL COMPYI(MAI, K-1, A(MUI,C1AI), LDA, SK, W,
     *                           YI, IERR)
                     IF (IERR .NE. 0) RETURN
                  ELSE
                     CALL DLAVEC(K-1, ZERO, YI, 1)
                  END IF
                  YI(K) = -ONE
                  CALL COMPTY(NP, NA, I, K, YI, MU, A, LDA, S, Y, LDY,
     *                        SK(NP+1), W, IERR)
                  IF (IERR .NE. 0) THEN
                     IERR = 3
                     RETURN
                  END IF
                  CALL COMPTV(MA, NA, I, IR, A, LDA, S, Y, LDY, W)
C
C                 Append the first NP by I block of Y to the unimodular
C                 U, and the last MP by I block to the column reduced R
C                 and the last MP elements of the I-th column of Y to
C                 the leading column coefficient matrix GAMC.
C
                  NNV = NNV + 1
                  DUR1 = 0
                  DO 80 KK = 1, I
                     NORM = DNRM2(NP, Y(1,KK), 1)
                     IF (NORM .GE. TOL) THEN
                        DUR1 = KK - 1
                        CALL DCOPY(NP, Y(1,KK), 1, U(1,NNV,KK), 1)
                     ELSE
                        CALL DLAVEC(NP, ZERO, U(1,NNV,KK), 1)
                     END IF
   80             CONTINUE
                  DU = MAX(DU, DUR1)
                  IND = NA - MP + 1
                  DO 90 KK = 1, B
                     NORM = DNRM2(MP, Y(IND,KK), 1)
                     IF (NORM .NE. ZERO) THEN
                        IERR = 4
                        RETURN
                     END IF
   90             CONTINUE
                  DUR1 = -1
                  DO 100 KK = B + 1, I
                     NORM = DNRM2(MP, Y(IND,KK), 1)
                     IF (NORM .GE. TOL) THEN
                        DUR1 = KK - B - 1
                        CALL DCOPY(MP, Y(IND,KK), 1, R(1,NNV,KK-B), 1)
                     ELSE
                        CALL DLAVEC(MP, ZERO, R(1,NNV,KK-B), 1)
                     END IF
  100             CONTINUE
                  IF (DUR1 .NE. -1) THEN
                     NCR1 = NCR1 + 1
                     ZERCOL(NNV) = .FALSE.
                     DR = MAX(DR, DUR1)
                     CALL DCOPY(MP, Y(IND,DUR1+B+1), 1, GAMC(1,NCR1), 1)
C
C                    Check whether R is still column reduced.
C
                     COLRDC = CKGAMC(MP, NCR1, GAMC, LDG, Q, LDQ, W,
     *                               TOL)
                  ELSE
                     ZERCOL(NNV) = .TRUE.
                  END IF
               END IF
               K = K + 1
               GO TO 70
            END IF
C           END WHILE 70
C
C           Save the transformed A if I = B.
C
            IF (I .EQ. B) THEN
               CALL DLACPY('G', MA+1, NA, A, LDA, AB, LDAB)
               NNVB = NNV
               DUB = DU
            END IF
C
            IF (NNV.LE.NP .AND. COLRDC) THEN
C
C              If i > b + dp + 1, then the degree of the next computed
C              column in R(s) will be greater than dp. This will occur
C              if a null vector is not found, due to rounding errors.
C
               COLRDC = I .LE. (B + DP + 1)
               I = I + 1
               MU(I) = MU(I-1) + MAI
            END IF
            GO TO 60
         END IF
C        END WHILE 60
         GO TO 20
      END IF
C     END WHILE 20
C
      IF (.NOT. COLRDC) IERR = 2
      RETURN
C *** Last line of COLRD1 ***
      END

