      SUBROUTINE DLARFV( N, V, INCV, TAU, ALPHA, X, INCX )
C
C  -- LAPACK like auxiliary routine
C     December 8, 1994
C
C     .. Scalar Arguments ..
      DOUBLE PRECISION   ALPHA, TAU
      INTEGER            INCX, INCV, N
C     .. Array Arguments ..
      DOUBLE PRECISION   X( * ), V( * )
C     ..
C
C  Purpose
C  =======
C
C  DLARFV performs a Householder reflection given by
C
C     ( alpha ) = H * ( alpha ) ,
C     (   x   )       (   x   )
C
C  where the orthogonal matrix H is given in the form
C
C     H = I - tau * ( 1 ) * ( 1 v') ,
C                   ( v )
C
C  where tau is a real scalar and v is a real (n-1)-element.
C  If tau is zero then H is assumed to be the unit matrix and the
C  transformation is skipped, otherwise tau must be in the range
C  ( 1.0, 2.0 ). tau and v will usually be supplied by routine DLARFG.
C
C  NB. This routine is derived from the Nag Fortran 77 O( n ) basic
C      linear agebra routine F06FUF.
C
C     .. Parameters ..
      DOUBLE PRECISION   ZERO
      PARAMETER          ( ZERO = 0.0D+0 )
C     .. Local Scalars ..
      DOUBLE PRECISION   BETA
C     .. External Functions ..
      DOUBLE PRECISION   DDOT
      EXTERNAL           DDOT
C     .. External Subroutines ..
      EXTERNAL           DAXPY
C     ..
C     .. Executable Statements ..
      IF( TAU.NE.ZERO ) THEN
         BETA  = ALPHA + DDOT( N, V, INCV, X, INCX )
         ALPHA = ALPHA - TAU * BETA
         CALL DAXPY( N-1, -TAU * BETA, V, INCV, X, INCX )
      END IF
C
      RETURN
C
C     End of DLARFV
C
      END
