C
      SUBROUTINE MKPENC(MP, NP, DP, P, LDP1, LDP2, A, LDA, MA, NA)
C
C     PURPOSE
C
C     Given an MP x NP polynomial matrix of degree dp
C                                    dp-1            dp
C     P(s) = P(0) + ... + P(dp-1) * s     + P(dp) * s            (1)
C
C     the subroutine MKPENC constructs the first degree part A of the
C     linearization of the polynomial matrix P(s), where
C
C            | P(dp)    0  .        .   0 |
C            | P(dp-1)  I  0        .   0 |
C        A = |   .      0  I  .     .   0 |                      (2)
C            |   .            .  .  .   . |
C            |   .               I  0   0 |
C            | P(0)     0           I   0 |
C
C     REMARK: This auxiliary routine is intended to be called only from
C             the routine COLRED.
C
C     ARGUMENTS IN
C
C        MP - INTEGER.
C            The number of rows of the polynomial matrix P(s).
C            MP >= 1.
C        NP - INTEGER.
C            The number of columns of the polynomial matrix P(s).
C            NP >= 1.
C        DP - INTEGER.
C            The degree of the polynomial matrix P(s).
C            DP >= 1.
C        P - DOUBLE PRECISION array of DIMENSION (LDP1,LP2,DP+1).
C            The leading MP by NP by (DP+1) part of this array must
C            contain the coefficients of the polynomial matrix P(s).
C            Specifically, P(i,j,k) must contain the coefficient of
C            s**(k-1) of the polynomial which is the (i,j)-th element
C            of P(s), where i = 1,2,...,MP, j = 1,2,...,NP and
C            k = 1,2,...,DP+1.
C        LDP1 - INTEGER.
C            The leading dimension of array P as declared in the calling
C            program.
C            LDP1 >= MP.
C        LDP2 - INTEGER.
C            The second dimension of array P as declared in the calling
C            program.
C            LDP2 >= NP.
C
C     ARGUMENTS OUT
C
C        A - DOUBLE PRECISION array of DIMENSION (LDA,(DP+1)*MP+NP).
C            The leading (DP+1)*MP by (DP+1)*MP+NP part of this array
C            contains the matrix A as described in (2).
C        LDA - INTEGER.
C            The leading dimension of array A as declared in the calling
C            program.
C            LDA >= (DP+1)*MP.
C        MA - INTEGER.
C            The number of rows of matrix A.
C        NA - INTEGER.
C            The number of columns of matrix A.
C
C     CONTRIBUTOR
C
C        A.J.Geurts.
C
C     REVISIONS
C
C        1992, October 27.
C        1994, December 8.
C
      IMPLICIT NONE
C     .. Parameters ..
      DOUBLE PRECISION ZERO, ONE
      PARAMETER (ZERO = 0.0D0, ONE = 1.0D0)
C     .. Scalar Arguments ..
      INTEGER MP, NP, DP, LDP1, LDP2, LDA, MA, NA
C     .. Array Arguments ..
      DOUBLE PRECISION P(LDP1,LDP2,*), A(LDA,*)
C     .. Local Scalars ..
      INTEGER J, M1, NJ
C     .. External Subroutines/Functions ..
      EXTERNAL DLACPY, DLASET
C
C     .. Executable Statements ..
C
C     Initialisation of the matrix A.
C
      M1 = DP * MP
      MA = M1 + MP
      NA = MA + NP
      CALL DLASET('G', MP, MA, ZERO, ZERO, A(1,NP+1), LDA)
      CALL DLASET('G', M1, MA, ZERO, ONE, A(MP+1,NP+1), LDA)
C
C     Insert the matrices P(0), P(1), ..., P(pd) in array A.
C
      NJ = M1 + 1
      DO 20 J = 1, DP + 1
         CALL DLACPY('G', MP, NP, P(1,1,J), LDP1, A(NJ,1), LDA)
         NJ = NJ - MP
   20 CONTINUE
C
      RETURN
C *** Last line of MKPENC ***
      END

